@@ -10,6 +10,11 @@ from networktools.colorprint import bprint, gprint, rprint
 from datadbs.general import GeneralData
 from asyncio import shield, wait_for
 from rethinkdb.errors import RqlDriverError, RqlError
+from networktools.time import timestamp, now
+
+from rich import print 
+import sys
+VERSION = sys.version_info
 
 
 class Rethink_DBS(GeneralData):
@@ -27,12 +32,13 @@ class Rethink_DBS(GeneralData):
         self.set_defaultdb(self.dbname)
         self.active_conn = False
         self.done_conn = False
-        r = rdb
-        r.set_loop_type('asyncio')
+        rdb.set_loop_type('asyncio')
         self.client_host = None
         self.client_port = 0
         self.session = None
-        self.r = r
+        self.r = rdb
+        self.reader = None 
+        self.writer = None
 
     def __repr__(self):
         return f"Rethink_DBS({self.default_db}, {self.address})"
@@ -44,13 +50,19 @@ class Rethink_DBS(GeneralData):
         if self:
             self.save_log(msg, level)
 
+
+    def __del__(self):
+        loop = asyncio.get_event_loop()
+        asyncio.run(self.close())
+        
+
     @property
     def client_address(self):
         return (self.client_host, self.client_port)
 
     @property
     def active(self):
-        return self.client_host and self.client_port>0
+        return self.client_host and self.client_port > 0
   
     def clean_client(self):
         self.client_host = None
@@ -68,15 +80,17 @@ class Rethink_DBS(GeneralData):
                 dbname = self.dbname
                 dbhost = self.address[0]
                 dbport = self.address[1]
-                self.session = await self.r.connect(db=dbname,
+                self.session, self.reader, self.writer = await self.r.connect(db=dbname,
                                                     host=dbhost,
                                                     port=dbport,
                                                     **kwargs)
 
+
                 self.client_host = self.session.client_address()
                 self.client_port = self.session.client_port()
                 self.done_conn = True
                 self.active_conn = True
+
                 self.logger.info(
                     f"Connected to RethinkDB with database {self.default_db}")
             else:
@@ -111,7 +125,13 @@ class Rethink_DBS(GeneralData):
     async def close(self):
         if self.session:
             await self.session.close(noreply_wait=False)
+            await self.writer.wait_closed()
+            del self.session
             self.session = None
+
+        if self.writer:
+            self.writer.close()
+            await self.writer.wait_closed()
         self.clean_client()
 
     async def reconnect(self, wait=False):
@@ -296,7 +316,8 @@ class Rethink_DBS(GeneralData):
 
     async def save_data(self, table_name, data, options=None):
         task = asyncio.current_task()
-        task.set_name(f"save_data_{table_name}")
+        if VERSION.major == 3 and VERSION.minor >= 8:
+            task.set_name(f"save_data_{table_name}")
         try:
             if table_name in self.tables[self.default_db]:
                 task_coro = self.r.db(self.defaultdb).table(table_name).insert(
@@ -379,7 +400,7 @@ class Rethink_DBS(GeneralData):
         lambda user: user['age]==30
         """
         task = asyncio.current_task()
-        if task:
+        if task and (VERSION.major == 3 and VERSION.minor >= 8):
             task.set_name(f"get_data_filter_{table_name}")
         dbname = self.dbname
         index = filter_opt.get('index')
